#!/usr/bin/env python
# -*- coding: UTF-8 -*-
u"""
utilitaire d'instanciation ligne de commande de fichiers templates à partir d'un dictionnaire xml

usage::

    %prog -x <path>/eole.xml -o <path>/test.txt -i <path>/dico.eol testtemplate.tmpl

testtemplate est le fichier à instancier

"""
from optparse import OptionParser

import os.path as osp
import sys
from os import rename as mv

from creole.cfgparser import EoleDict
from creole.template import Template

# la position doit être le répertoire courant
# os.getcwd()
# DATADIR = osp.join(osp.dirname(__file__))
# DATADIR=pwd()

def instance(inputfile, xmlfile, outputfile, inifile):
    """
    renommer le fichier source en un .ori
    car le template va le copier au mme endroit
    """
    datadir = osp.join(osp.dirname(inputfile))
    t = Template(inputfile, templatedir=datadir)
    eoledict = EoleDict(templatedir=datadir)
    if osp.isdir(xmlfile):
        eoledict.read_dir(xmlfile)
    else:
        eoledict.read(xmlfile)
    eoledict.load_values(inifile)
    t.target = outputfile
    t.process(eoledict)

def parse_cmdline():
    parser = OptionParser(usage=__doc__)
    parser.add_option("-x", "--xml", dest="xmlfile",
                  help=u"xml config file", metavar="XMLFILE")

    parser.add_option("-o", "--output", dest="outputfile",
                  help=u"instanciated output file", metavar="OUTPUTFILE")

    parser.add_option("-i", "--input", dest="inifile",
                  help=u"instanciated .ini file", metavar="INPUTTFILE")

    (options, args) = parser.parse_args()

    if not options.xmlfile:
        parser.error("il faut un fichier xml de configuration")
    if not options.outputfile:
        parser.error("il faut un nom de fichier de sortie")

    if not len(args) == 1:
        parser.error("il faut un et un seul fichier de template")

    if args[0] == options.outputfile:
        parser.error("il faut un nom de fichier different")

    return options, args

def parse():
    options, args = parse_cmdline()
    inputfile = args[0]
    xmlfile = options.xmlfile
    outputfile = options.outputfile
    inifile = options.inifile
    return inputfile, xmlfile, outputfile, inifile

def main():
    instance(*parse())

if __name__ == '__main__':
    main()
