#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
utilitaire de conversion de templates creole1 vers creole2

usage::

    %prog <-m nom_module> -o fichier_creole2 fichier_creole1
"""
from optparse import OptionParser
import os.path as osp
import sys, re

from creole import config, dico
from creole.fonctionseole import get_module_name

def parse_cmdline():
    parser = OptionParser(usage=__doc__)
    parser.add_option("-m", "--module", dest="module", default=get_module_name(),
                  help=u"nom du module eole", metavar="MODULE")

    parser.add_option("-o", "--output", dest="outputfile",
                  help=u"destination file", metavar="OUTPUTFILE")

    options, args = parser.parse_args()

    if not options.outputfile:
        parser.error("il faut un nom de fichier de sortie")
    if not len(args) == 1:
        parser.error("il faut un et un seul fichier source")
    if args[0] == options.outputfile:
        parser.error("il faut un nom de fichier different")

    return options, args

def convert_tmpl(module, inputfile, outputfile):
    dict_convert = {}
    try:
        dict_ori = dico.read_convert(osp.join(config.datadir, 'convert', '%s.csv' % module))
    except:
        # le module demandé n'est pas géré
        pass
    else:
        for key, values in dict_ori.items():
            # si une variable creole2 multiple correspond à plusieurs variables creole1
            # on remplace les 2 par la variable creole2 (on ne gère pas les itérations)
            for cle in values:
                dict_convert[cle] = key
    # conversion des variables creole1 -> 2
    data = file(inputfile).read()
    for old_var, new_var in dict_convert.items():
        data = data.replace('%%'+old_var+'%%','%%'+new_var)
    not_replaced = re.findall('%%[\w*]+%%',data)
    if not_replaced != []:
        print "!!les variables suivantes n'ont pas été remplacées !!\n"
        print "\n".join(not_replaced)
        print "\n"
    # écriture du fichier de sortie
    f_out = file(outputfile,'w')
    f_out.write(data)
    f_out.close()
    return not_replaced

def main():
    options, args = parse_cmdline()
    inputfile = args[0]
    outputfile = options.outputfile
    module = options.module
    convert_tmpl(module, inputfile, outputfile)

if __name__ == '__main__':
    main()
