# -*- coding: UTF-8 -*-
from creole import ifconfig
from creole.error import DependencyError
import commands

def _create_ip(ip_list):
    template = """%i.%i.%i.%i"""
    return template%tuple(ip_list)

def _create_ip_list(ip):
    a,b,c,d=ip.split('.')
    return [int(a),int(b),int(c),int(d)]

def calc_network_auto(ip, mask):
    """calcul l'adresse de broadcast à partir de l'ip et du netmask
    """
    mask_list = _create_ip_list(mask)
    ip_list = _create_ip_list(ip)
    network_list = []
    for i in range(4):
        #pour l'adresse réseau c'est simple
        network_list.append(int(ip_list[i]) & int(mask_list[i]))
    network = _create_ip(network_list)
    return network

def server_mem():
    """renvoit la mémoire du serveur
    """
    procmem = open('/proc/meminfo')
    meminfo = procmem.read().split('\n')
    procmem.close() 
    for line in meminfo:
        if line.startswith('MemTotal'):
            mem_tot = float(line.split()[1])
            break
    return str(mem_tot)

def kernel_version():
    """ renvoit la version du kernel
    """
    return_code, output = commands.getstatusoutput('uname -r')
    if return_code ==0:
        return str(output)
    else:
        return ''

def auto_eth(nom_carte, **kw):
    """ renvoit l'adresse de la carte nom_carte
    """
    if kw['condition'].lower() == kw['parametre'].lower():
        eths = ifconfig.ifconfig()
        if nom_carte in eths.getInterfaceList():
            eth = eths.getAddr(nom_carte)
        else:
            return ''
        return str(eth)
    else: 
        raise DependencyError

def auto_netmask(nom_carte, **kw):
    """ renvoit le netmask pour la carte nom_carte
    """
    if kw['condition'].lower() == kw['parametre'].lower():
        eths = ifconfig.ifconfig()
        if nom_carte in eths.getInterfaceList():
            netmask = eths.getNetmask(nom_carte)
        else:
            return ''
        return str(netmask)
    else:
        raise DependencyError

def auto_network(nom_carte, **kw):
    """ renvoit le network pour la carte nom_carte
    """
    if kw['condition'].lower() == kw['parametre'].lower():
        eths = ifconfig.ifconfig()
        if nom_carte in eths.getInterfaceList():
            ip = eths.getAddr(nom_carte)
            netmask = eths.getNetmask(nom_carte)
            network = calc_network_auto(ip, netmask)
        else:
            return ''
        return str(network)
    else:
        raise DependencyError

def auto_broadcast(nom_carte, **kw):
    """ renvoit le broadcast pour la carte nom_carte
    """
    if kw['condition'].lower() == kw['parametre'].lower():
        eths = ifconfig.ifconfig()
        if nom_carte in eths.getInterfaceList():
            broadcast = eths.getBroadcast(nom_carte)
        else:
            return ''
        return str(broadcast)
    else:
        raise DependencyError


