# -*- coding: UTF-8 -*-

from creole import config
from creole.error import FileNotFound
import os ,shutil, time, sys, re, signal
import glob

def patch():
    """
    applique les patchs 
    """
    distrib_dir = config.distrib_dir
    patch_dir = config.patch_dir
    template_dir = config.templatedir
    var_dir = os.path.join(patch_dir,'variante')
    # sauvegarde du répertoire de départ
    cur_dir = os.getcwd()
    os.chdir(template_dir)
    # copie des templates originaux
    os.system('cp -f %s/* %s' % (distrib_dir, template_dir))
    # patches variante + locaux
    for dir in [var_dir, patch_dir]:
        patches = os.listdir(dir)
        for patch in patches:
            if patch.endswith('.patch'):
                os.system('patch -N -p1 < %s >> %s' % (os.path.join(dir, patch), config.logfile))
    os.chdir(cur_dir)

def activate_services():
    """
    active les services
    """
    # services disponibles 
    tserv = open(config.tserv)
    services_dispo = tserv.read().strip().split('\n')
    tserv.close()
    fserv = open(config.fserv)
    services_actifs = fserv.read().strip().split('\n')
    fserv.close()
    for service in services_dispo:
        if not service.startswith('%'):
            if service in services_actifs:
                os.system('/usr/sbin/update-rc.d %s defaults >/dev/null' % service)
            else:
                os.system('/etc/init.d/%s stop >/dev/null' % service)
                os.system('/usr/sbin/update-rc.d -f %s remove >/dev/null' % service)

def restart_services(ead=False, ead2=False):
    """
    redémarre les services actifs
    """
    print "** Redémarrage des services **"
    print ""
    fserv = open(config.fserv)
    services = fserv.read().strip().split('\n')
    fserv.close()
    for service in services:
        if ead == True and service in ['apache2']:
            os.system('/etc/init.d/%s reload' % service)
        elif ead2 == True and service in ['ead-server', 'ead-web']:
            pass
        else:
            os.system('/etc/init.d/%s restart' % service)

def get_module_name(version=False):
    """
    renvoie le nom du module
    @param version: renvoie aussi la version si True
    """
    if not os.path.isfile(config.f_version):
        raise FileNotFound, '%s non trouvé' % config.f_version
    f_vers = open(config.f_version)
    content = f_vers.read().strip()
    f_vers.close()
    if content == '':
        raise ConfigError, '%s vide' % config.f_version
    if version:
        return content
    else:
        return content.split('-')[0]

def controle_kernel():
    """
    renvoie True si il faut rebooter (nouveau kernel)
    """
    grub_index = -1
    NoyauActuel = os.popen("uname -r", "r").readline().strip()
    try:
        grub_default = open("/boot/grub/default").read().split('\n')[0].strip()
        current_grub_index = int(grub_default.replace('\x00',''))
    except:
        print "\n!! erreur lors de la lecture de /boot/grub/default !!\n"

    boot_ok = 1
    if os.path.exists(config.REBOOT_FILE):
        boot_ok = 0
    if os.path.exists(config.KERNEL_FILE):
        # on utilise le noyau spécifié
        new_index = -1
        DernierNoyau = open(config.KERNEL_FILE).read().strip()
        if DernierNoyau != NoyauActuel:
            grub_index = parse_menu_lst(DernierNoyau)
            if grub_index != -1:
                new_index = grub_index
        else:
            grub_index = parse_menu_lst(DernierNoyau)
            if current_grub_index != grub_index:
                # cas ou un nouveau noyau a été installé (mais pas le bon)
                new_index = grub_index
            if grub_index == -1:
                print "\n!! noyau non trouvé : %s !!\n" % DernierNoyau
        if new_index != -1:
            res = os.system('grub-set-default %s' % grub_index)
            if res != 0:
                print "\n!! erreur lors de la selection du noyau par défaut (index %s) !!\n" % grub_index
                return False
    else:
        DernierNoyau  = os.popen("ls -l /vmlinuz ", "r").readline().strip().split("boot/vmlinuz-")[1]
        if DernierNoyau != NoyauActuel:
            return True
    if boot_ok == 0:
        return True
    return False

def parse_menu_lst(DernierNoyau):
    menu = open('/boot/grub/menu.lst','r')
    express = "^title[ \t]+(.*)$"

    lines = menu.readlines()
    menu.close()
    kernels = []
    for i in lines:
         match = re.search(express, i)
         if match:
             kernels.append(match.group(1))

    n=0
    kernels_dict = {}
    while n < len(kernels):
        if not '(recovery mode)' in kernels[n] and 'kernel' in kernels[n]:
            kernels_dict[kernels[n].split('kernel')[1].strip()]=n
        n += 1
    if DernierNoyau in kernels_dict.keys():
        return kernels_dict[DernierNoyau]
    else:
        return -1

def update_mac():
    """
    mise à jour des adresses MAC dans les fichiers ethX.dev
    """
    mac_str = 'MAC='
    dmac_str = 'DEVICE_MAC='
    cmd = os.popen('/sbin/ifconfig -a | grep HWaddr')
    macs = cmd.read().split('\n')
    cmd.close()
    net_files = glob.glob('/etc/sysconfig/network/eth*.dev')
    for net_file in net_files:
        eth = os.path.basename(net_file).split('.')[0]
        for mac in macs:
            if mac.startswith(eth):
                print "Sauvegarde de l'adresse MAC pour %s\n" % eth
                conf_file = open(net_file,"r")
                buffer_in = conf_file.readlines()
                conf_file.close()
                buffer_out = ""
                for line in buffer_in:
                    if line.startswith(mac_str):
                        buffer_out += "%s'%s'\n" % (mac_str, mac.split()[4])
                    elif line.startswith(dmac_str):
                        buffer_out += "%s'%s'\n" % (dmac_str, mac.split()[4])
                    else:
                        buffer_out += line
                conf_file = open(net_file,"w")
                conf_file.write(buffer_out)
                conf_file.close()
                break

def zephir(etat, msg, type_proc):
    # selon l'etat donné, on indique l'état de la tache
    etat_zeph = None
    if etat.upper().startswith("INIT"):
        etat_zeph = -1
    elif etat.upper().startswith("FIN"):
        etat_zeph = 0
    elif etat.upper().startswith('ERR'):
        etat_zeph = 1
    elif etat.upper().startswith('MSG'):
        etat_zeph = -2
    # log local si msg ou erreur
    if (len(msg) > 0) or (etat.upper() == "ERR"):
        log(etat, msg, type_proc)
    # log sur zephir si disponible
    if etat_zeph is not None and config.zephir_actif:
        config.log(type_proc,etat_zeph,msg,str(time.ctime()))

def log(etat,msg,type_proc):
    """
    effectue un log local et éventuellement sur zephir
    """
    if etat == "ERR":
        sys.stderr.write("\n%s : Erreur => %s\n" % (type_proc, msg))
    # log local
    os.system("""/usr/bin/logger -t \"zephir\" -p local2.err  \"%s => %s : %s \" """ % (type_proc, etat, msg))
     
def init_proc(type_proc):
    """
    initialisation d'une procédure (log démarrage + bcage éventuel)
    """
    if  verify_lock(type_proc):
        return True
    else:
        return False

#def end_proc(etat,msg,type_proc):
#    """
#    loggue la fin d'une procédure
#    """
#     log(etat,msg,type_proc )

def verify_lock(name):
    """
    vérifie le bloquage ou non d'une  procédure
    """
    LOCK_FILE= "/usr/share/zephir" + os.sep + "zephir_locks"
    if name == "":
        return True
    from zephir.lib_zephir import zephir_path, zephir_dir
    try:
        from zephir.lib_zephir import config, zephir, convert
        locks = convert(zephir.serveurs.get_locks(config.id_serveur))
        if locks[0] == 0:
            # erreur sur zephir, on ignore cette phase
            raise Exception
        locks=[lock[0] for lock in locks[1]]
    except Exception, e:
        # pas de message d'erreur si le serveur n'est pas enregistré
        if os.path.isfile(os.path.join(zephir_path,"zephir_conf","zephir_conf.py")):
            # on ne bloque pas si l'appel à zephir échoue
            print "erreur d'accès à zephir : %s" % str(e)
        # on regarde le denier état
        if os.path.exists(LOCK_FILE):
            file_lock = file(LOCK_FILE)
            locks=file_lock.read().split('\n')
            file_lock.close()
            # on bloque si interdit
            if name in locks:
                return False
    else:
        # mise en place du fichier de droits
        content = "\n".join(locks)
        try:
            file_lock = file(LOCK_FILE,"w")
            file_lock.write(content)
            file_lock.close()
        except:
            print "impossible de mettre à jour %s (droits insuffisants)" % LOCK_FILE
            pass
        # retour du code
        if name in locks:
            return False
    return True


def register_ead(dico):
    """
    Enregistrement de l'EAD local
    """
    if dico['adresse_ip_eth0'] == '':
        return
    from os.path import join
    sys.path.insert(1, '/usr/share')
    try:
        from ead2.frontend.web.lib.tools import servers
        from ead2.backend.lib.eadserver import EadKeyParser
        from ead2.backend.lib.eadserver import create_magicnb
        from ead2.backend.config.config import CONFIG_DIR, FRONTEND_KEYS_FILE
        from ead2.config.config import BACKEND_LISTEN_PORT
    except:
        return
    e = EadKeyParser()
    e.parse_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE))
    if e.get_key_dict() != {}:
        # des frontend sont déjà enregistrés
        return
    # FIXME : Amon
    key = create_magicnb('admin','admin')
    d = {dico['adresse_ip_eth0'] : key}
    # ecriture clé ead-serveur
    e.write_file(join(CONFIG_DIR, FRONTEND_KEYS_FILE), d)
    # ecriture clé ead-web
    servers().add_server('https://%s'%dico['adresse_ip_eth0'], str(BACKEND_LISTEN_PORT), dico['nom_machine'], str(key))

def register_module(ask=False):
    """
    Enregistrement Eole
    """
    tool_dir = '/usr/share/register'
    if not os.path.isdir(tool_dir):
        # eole-register pas installé ?
        return 
    if not os.path.isfile(os.path.join(tool_dir,'.registerok')):
        # pas d'enregistrement préalable
        if not ask:
            return
        print
        print " http://eole.ac-dijon.fr/materiel/ vous permet de consulter une base des matériels compatibles"
        print "Pour enrichir cette base, acceptez-vous l'envoi de la description matérielle de ce serveur (oui/non) ?"
        rep = raw_input("[oui] : ")
        if rep.lower() in ['non','n']:
            return
        file(os.path.join(tool_dir,'.registerok'),'w').close()

    sys.path.append('/usr/share/')
    from register import register
    os.chdir(tool_dir)
    register.initialize()

def proper_exit(*args):
    """callback à appeler lors d'un ctrl-c"""
    print("Abandon utilisateur")
    os._exit(1)

def catch_signal():
    """gestion du ctrl-c"""
    signal.signal(signal.SIGINT,proper_exit)

